<?php

namespace Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use \Teknisa\GER\Services\SMTPService;
use \Teknisa\Model\Mymenu\Entities;

class Mym00111UsuariosMyMenuController extends \Teknisa\GER\Controllers\TeknisaCrud
{

    protected $dataSourceName = 'unidade_servico_usuario';

    protected $crudService;
    protected $parameterBag;
    protected $entityManager;
    protected $newCode;
    protected $environment;
    protected $smtpService;
    protected $paramsSMTP = array(
        'host' => "smtp.teknisa.com",
        'username' => "app@tecfood.com",
        'password' => "@AppTecFood2015",
    );

    public function __construct(
        $crudService, $parameterBag, $entityManager, $newCode,
        \Teknisa\GER\Utilities\Environment $environment, SMTPService $smtpService, $dataSourceManager 
    ) {
        parent::__construct($crudService, $parameterBag);
        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->smtpService = $smtpService;
        $this->templatesDir = __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'Templates' . DIRECTORY_SEPARATOR;
    }

    public function salvaNovoUsuarioMyMenu(
        DTO\Request\Row $request, DTO\Response $response
    ) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $rows = $request->getRow()[0];

            $nrOrg = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            $data = $this->environment->getCurrentTime();

            $codigoUsuario = $this->salvaUsuario(
                $rows['usuario'], $nrOrg, $operador, $data);

            $codigoAssociacao = $this->salvaAssocUsuarioAplicativo(
                $codigoUsuario, $nrOrg, $operador, $data);

            $connection->commit();
            return true;
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function salvaUsuario($usuario, $nrOrg, $operador, $data)
    {
        try {
            /** @var Entities\Usuarioapp $dadosUsuario */
            $dadosUsuario = $this->entityManager->getRepository(
                '\Teknisa\Model\Mymenu\Entities\Usuarioapp'
            )
                ->findOneBy(
                    array(
                        'nrorg' => $nrOrg,
                        'dsemailusuarioapp' => $usuario['DSEMAILUSUARIOAPP'],
                    )
                );

            if ($dadosUsuario instanceof Entities\Usuarioapp) {
                $codigo = $dadosUsuario->getNrusuarioapp();
            } else {
                $codigo = $this->newCode->getCodeByTableName(
                    "USUARIOAPP", $nrOrg
                );

                $usuarioapp = new Entities\Usuarioapp();
                $usuarioapp->setNrusuarioapp($codigo);
                $usuarioapp->setDsemailusuarioapp(
                    $usuario['DSEMAILUSUARIOAPP']
                );
                $usuarioapp->setDsprimeironome($usuario['DSPRIMEIRONOME']);
                $usuarioapp->setDsultimonome($usuario['DSULTIMONOME']);
                $usuarioapp->setDstelefone($usuario['DSTELEFONE']);
                $usuarioapp->setIdativo('S');
                $usuarioapp->setNrorg($nrOrg);
                $usuarioapp->setNrorginclusao($nrOrg);
                $usuarioapp->setCdoperinclusao($operador);
                $usuarioapp->setDtinclusao($data);

                $this->entityManager->persist($usuarioapp);
                $this->entityManager->flush();
            }

            return $codigo;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvaAssocUsuarioAplicativo(
        $codigoUsuario, $nrOrg, $operador, $data
    ) {
        try {
            /** @var Entities\AppUsuarioaplicativo $dadosUsuario */
            $dadosUsuario = $this->entityManager->getRepository(
                '\Teknisa\Model\Mymenu\Entities\AppUsuarioaplicativo'
            )
                ->findOneBy(
                    array(
                        'nrorg' => $nrOrg,
                        'nrusuarioapp' => $codigoUsuario,
                        'nraplicativo' => 1
                    )
                );

            if ($dadosUsuario instanceof Entities\AppUsuarioaplicativo) {
                $codigo = $dadosUsuario->getNrusuarioaplicativo();
            } else {
                $codigo = $this->newCode->getCodeByTableName(
                    "APP_USUARIOAPLICATIVO", $nrOrg
                );

                $assocUsuarioAplic = new Entities\AppUsuarioaplicativo();
                $assocUsuarioAplic->setNrusuarioaplicativo($codigo);
                $assocUsuarioAplic->setNrusuarioapp($codigoUsuario);
                $assocUsuarioAplic->setNraplicativo(1);
                $assocUsuarioAplic->setIdativo('S');
                $assocUsuarioAplic->setNrorg($nrOrg);
                $assocUsuarioAplic->setNrorginclusao($nrOrg);
                $assocUsuarioAplic->setCdoperinclusao($operador);
                $assocUsuarioAplic->setDtinclusao($data);

                $this->entityManager->persist($assocUsuarioAplic);
                $this->entityManager->flush();
            }

            return $codigo;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function salvaUnidadeUsuarioComServico(
        $codigoAssociacao, $unidadeServico, $nrOrg, $operador, $data
    ) {
        try {
            $unidades = $unidadeServico['CDFILIAL'];
            $servicos = $unidadeServico['CDSERVFILI'];
            $servicosTratados = $this->trataServicos($unidades, $servicos);

            $servico = explode("_", $servicosTratados);
            
            $unidade = "";
            foreach ($servico as $srv) {
                if($unidade != substr($srv,0,4)){
                    $unidade = substr($srv,0,4);
                    /** @var Entities\AppUsuarioclifil $usuarioCliFil */
                    $usuarioCliFil = $this->entityManager->getRepository(
                        '\Teknisa\Model\Mymenu\Entities\AppUsuarioclifil'
                    )
                        ->findOneBy(
                            array(
                                'nrorg' => $nrOrg,
                                'nrusuarioaplicativo' => $codigoAssociacao,
                                'cdfilial' => $unidade
                            )
                        );
                    
                    if ($usuarioCliFil instanceof Entities\AppUsuarioclifil) {
                        $codigoUsCliFil = $usuarioCliFil->getNrusuarioclifil();
                    } else {
                        $codigoUsCliFil = $this->newCode->getCodeByTableName(
                            "APP_USUARIOCLIFIL", $nrOrg
                        );
                        
                        $assocUsuarioFil = new Entities\AppUsuarioclifil();
                        $assocUsuarioFil->setNrusuarioclifil($codigoUsCliFil);
                        $assocUsuarioFil->setNrusuarioaplicativo(
                            $codigoAssociacao
                        );
                        $assocUsuarioFil->setIdselecionado('N');
                        $assocUsuarioFil->setCdfilial($unidade);
                        $assocUsuarioFil->setIdativo('S');
                        $assocUsuarioFil->setNrorg($nrOrg);
                        $assocUsuarioFil->setNrorginclusao($nrOrg);
                        $assocUsuarioFil->setCdoperinclusao($operador);
                        $assocUsuarioFil->setDtinclusao($data);
    
                        $this->entityManager->persist($assocUsuarioFil);
                        $this->entityManager->flush();
                    }
                }
                
                $conditions = array(
                    array(
                        "columnName" => "NRUSUARIOCLIFIL",
                        "operator" => "=",
                        "value" => $codigoUsCliFil
                    ),
                    array(
                        "columnName" => "NRORG",
                        "operator" => "=",
                        "value" => $this->environment->getNrOrgTrab()
                    ),
                    array(
                        "columnName" => "CDSERVFILI",
                        "operator" => "=",
                        "value" =>  substr($srv, 4)
                    )
                );
                
                $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('app_usuarioservfil', $conditions);
                $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                $usuarioServFil = $dataSet->getRows();                    

                if (empty($usuarioServFil)) {
                    $codigoUsuarioServFil = $this->newCode->getCodeByTableName(
                        "APP_USUARIOSERVFIL", $nrOrg
                    );

                    $dadosAppUsuServFil = array(
                        'NRORG' => $nrOrg,
                        '__is_new' => true,
                        'NRUSUARIOCLIFIL' => $codigoUsCliFil,
                        'NRUSUARIOSERVFIL' => $codigoUsuarioServFil,
                        'CDSERVFILI' => substr($srv, 4)
                    );
                    
                    $requestInsertAppUsuServFil = new DTO\Request\Row(
                        $dadosAppUsuServFil, '', '', ''
                    );
                    $dadosAppUsuServFil = $this->crudService->getRowsToSave(
                        $requestInsertAppUsuServFil, 'app_usuarioservfil'
                    );
                    
                    $this->crudService->save('app_usuarioservfil', $dadosAppUsuServFil);                            
                }
                
            }
            
            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function trataServicos($unidades, $servicos)
    {
        if ($servicos === 'T') {
            $dataSourceName = "servico_filial";
            $conditions = array(
                array(
                    "columnName" => "NRORG",
                    "operator" => '=',
                    "value" => $this->environment->getNrOrg(),
                ),
                array(
                    "columnName" => "CDFILIAL",
                    "operator" => '=',
                    "value" => $unidades,
                )
            );

            $retornaServicos = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
            if (count($retornaServicos->getRows()) > 0) {
                $servicos = implode('_', array_column($retornaServicos->getRows(), "CHAVE"));
            } else {
                $servicos = "";
            }
        }

        return $servicos;
    }

    public function incluiServFilMyMenu(
        DTO\Request\Row $request, DTO\Response $response
    ) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $rows = $request->getRow()[0];

            $nrOrg = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            $data = $this->environment->getCurrentTime();

            /** @var Entities\AppUsuarioaplicativo $usuarioAssocApp */
            $usuarioAssocApp = $this->entityManager->getRepository(
                '\Teknisa\Model\Mymenu\Entities\AppUsuarioaplicativo'
            )
                ->findOneBy(
                    array(
                        'nrorg' => $nrOrg,
                        'nrusuarioapp' => $rows['usuario'],
                        'nraplicativo' => 1
                    )
                );

            $codigoAssociacao = $usuarioAssocApp->getNrusuarioaplicativo();

            $this->salvaUnidadeUsuarioComServico(
                $codigoAssociacao, $rows['unidadeServico'],
                $nrOrg, $operador, $data);

            $connection->commit();
            return true;
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function excluiUsuarioMyMenu(
        DTO\Request\Row $request, DTO\Response $response
    ) {
        $codigoUsuario = $request->getRow()[0];
        $nrOrg = $this->environment->getNrOrg();

        try {
            // RETORNA NÚMERO DE APLICATIVOS CADASTRADOS
            /** @var Entities\AppUsuarioaplicativo $usuarioApp */
            $usuarioApps = $this->entityManager->getRepository(
                '\Teknisa\Model\Mymenu\Entities\AppUsuarioaplicativo'
            )
                ->findBy(
                    array(
                        'nrorg' => $nrOrg,
                        'nrusuarioapp' => $codigoUsuario,
                        'nraplicativo' => 1
                    )
                );

            foreach ($usuarioApps as $usuarioApp) {
                if ($usuarioApp->getNraplicativo() == 1) {
                    $cdUsuarioApp = $usuarioApp->getNrusuarioaplicativo();
                }

            }
            if(isset($cdUsuarioApp)){
                // EXCLUI SERVICOS
                $connection = $this->entityManager->getConnection();
                $query = "DELETE FROM APP_USUARIOSERVFIL
                           WHERE NRUSUARIOCLIFIL IN (SELECT NRUSUARIOCLIFIL
                                                       FROM APP_USUARIOCLIFIL
                                                       WHERE NRUSUARIOAPLICATIVO = :NRUSUARIOAPLICATIVO)";
                $connection->executeQuery(
                    $query, array(
                        "NRUSUARIOAPLICATIVO" => $cdUsuarioApp,
                        "NRORG" => $nrOrg,
                    )
                );
    
                // EXCLUI UNIDADES/CLIENTES
                $connection = $this->entityManager->getConnection();
                $query
                = "DELETE FROM APP_USUARIOCLIFIL
                        WHERE NRUSUARIOAPLICATIVO = :NRUSUARIOAPLICATIVO";
                $connection->executeQuery(
                    $query, array(
                        "NRUSUARIOAPLICATIVO" => $cdUsuarioApp,
                        "NRORG" => $nrOrg,
                    )
                );
    
                // EXCLUI ASSOCIAÇÃO AO MYMENU
                $connection = $this->entityManager->getConnection();
                $query = "DELETE FROM APP_USUARIOAPLICATIVO
                           WHERE NRUSUARIOAPP = :NRUSUARIOAPP
                             AND NRAPLICATIVO = 1";
                $connection->executeQuery(
                    $query,
                    array("NRUSUARIOAPP" => $codigoUsuario, "NRORG" => $nrOrg)
                );
            }
            $appUsuario = $this->entityManager->getRepository(
                '\Teknisa\Model\Mymenu\Entities\AppUsuarioaplicativo'
            )
                ->findBy(
                    array(
                        'nrorg' => $nrOrg,
                        'nrusuarioapp' => $codigoUsuario
                    )
                );
            
            if (!($appUsuario instanceof Entities\AppUsuarioaplicativo)) {
                // EXCLUI USUARIOAPP
                $connection = $this->entityManager->getConnection();
                $query = "DELETE FROM USUARIOAPP
                           WHERE NRUSUARIOAPP = :NRUSUARIOAPP";
                $connection->executeQuery(
                    $query, array("NRUSUARIOAPP" => $codigoUsuario,
                                  "NRORG" => $nrOrg
                            )
                ); 
            }
            return true;
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function excluiServFilUsuario(
        DTO\Request\Row $request, DTO\Response $response
    ) {
        $row = $request->getRow()[0];
        $nrOrg = $this->environment->getNrOrg();
        try {
            // EXCLUI SERVICOS
            $connection = $this->entityManager->getConnection();
            $query = "DELETE FROM APP_USUARIOSERVFIL
                       WHERE NRUSUARIOSERVFIL = :NRUSUARIOSERVFIL";
            $connection->executeQuery(
                $query, array(
                    "NRUSUARIOSERVFIL" => $row['NRUSUARIOSERVFIL'],
                    "NRORG" => $nrOrg
                )
            );

            $conditions = array(
                array(
                    "columnName" => "NRUSUARIOCLIFIL",
                    "operator" => "=",
                    "value" => $row['NRUSUARIOCLIFIL']
                ),
                array(
                    "columnName" => "NRORG",
                    "operator" => "=",
                    "value" => $nrOrg
                )
            );
            
            $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('app_usuarioservfil', $conditions);
            $dataSet = $this->dataSourceManager->findBy($filterCriteria);
            $usuarioServFil = $dataSet->getRows();                    

            if (empty($usuarioServFil)){

                // EXCLUI UNIDADES/CLIENTES
                $connection = $this->entityManager->getConnection();
                $query
                = "DELETE FROM APP_USUARIOCLIFIL
                        WHERE NRUSUARIOCLIFIL = :NRUSUARIOCLIFIL";
                $connection->executeQuery(
                    $query, array(
                        "NRUSUARIOCLIFIL" => $row['NRUSUARIOCLIFIL'],
                        "NRORG" => $nrOrg,
                    )
                );
            }
            return true;
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    //email
    public function fileGetContentUtf8($fileName)
    {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }

    public function buildInvitationEmailWithQrCode($qrcode, $name)
    {
        $content = $this->fileGetContentUtf8($this->templatesDir . "mail_invitation_with_qrcode.html");
        $labelAppName = "MyMenu";
        $qrcodeIMG = $qrcode['data'];
        $qrcodeURL = $qrcode['url'];

        $title = "Bem vindo ao Tecfood MyMenu";
        $mainInfo = 'Convido você!';

        $search = array(
            "%title%",
            "%destinatario%",
            "%data%",
            "%qrcodeURL%",
            "%qrcodeIMG%",
            "%labelAppName%",
        );
        $replace = array(
            $title,
            $name,
            $mainInfo,
            $qrcodeURL,
            $qrcodeIMG,
            $labelAppName,
        );
        return str_replace($search, $replace, $content);
    }

    public function buildInvitationEmail($name)
    {
        $content = $this->fileGetContentUtf8($this->templatesDir . "mail_invitation.html");
        $labelAppName = "MyMenu";

        $title = "Bem vindo ao Tecfood MyMyMenu";
        $mainInfo = 'Convido você!';

        $search = array(
            "%title%",
            "%destinatario%",
            "%data%",
            "%labelAppName%",
        );
        $replace = array(
            $title,
            $name,
            $mainInfo,
            $labelAppName,
        );
        return str_replace($search, $replace, $content);
    }

    public function sendInvitationEmail(DTO\Request $request, DTO\Response $response)
    {
        $row = (array) $request->getRow();
        $users = $row['selectedUsers'];
        $qrcode = $row['qrCode'];
        
        for ($i = 0; $i < count($users); $i++) {
            $curUser = $users[$i];

            $name = $curUser['DSPRIMEIRONOME'] . ' ' . $curUser['DSULTIMONOME'];
            $sendTo = $curUser['DSEMAILUSUARIOAPP'];
            
            try {

                $title = 'Convite MyMenu';
                $this->smtpService->setFrom('teknisa');
                $this->smtpService->setSubject($title);
                $this->smtpService->setFromName('SAC MyMenu');
                if($qrcode['url'] === 'nodns'){
                    $mailBody = $this->buildInvitationEmail($name);
                }else{
                    $mailBody = $this->buildInvitationEmailWithQrCode($qrcode, $name);
                }
                $this->smtpService->sendMail($mailBody, $sendTo, $title);
            } catch (\Exception $e) {
                $response->setError(new Error($e->getMessage(), 500));
            }
        }

    }
}
