<?php
namespace Folha\Entities;

use HCM\Util\DateUtil;
use Importacao\Logic\Integracao\ProjetoImportacao;

class MigVinculom extends Base\MigVinculom
{

    const DEFAULT_TP_ADMISSAO = '2';
    const DEFAULT_TP_VINCULO = '1';
    const DEFAULT_NRDIAS = '45';
    const EXPRESSAO_NUMEROS = '/[^0-9]/';

    protected $dtexamemedico;
    protected $dtopcaofgts;
    protected $dtrescisaovinc;
    protected $dtavisotrct;
    protected $dtprimadmissao;
    protected $dtadmissaovinc;
    protected $dtaposentafgts;
    protected $dtavisoprevio;
    protected $dtultatu;
    protected $dtinclusao;

    /**
     * @param $nrvinculom
     * @return \Folha\Entities\MigVinculom
     */
    public function setNrvinculom ($nrvinculom) {
        $nrvinculom = \Importacao\Logic\Validacoes::verificaChaveDuplicada('GPE_VINCULOM', 'NRVINCULOM', ProjetoImportacao::getNrorg(), $nrvinculom);
        $this->nrvinculom = $nrvinculom;
    }

    /**
     * @param $nrpessoa
     * @return \Folha\Entities\MigVinculom
     */
    public function setNrpessoa ($nrpessoa) {
        $nrpessoa = \Importacao\Logic\Validacoes::verificaChaveMae('GPE_PESSOA', 'NRPESSOA', ProjetoImportacao::getNrorg(), $nrpessoa);
        $this->nrpessoa = $nrpessoa;
    }

    /**
     * @param $nmvinculom
     * @throws \Exception
     * @return \Folha\Entities\MigVinculom
     */
    public function setNmvinculom ($nmvinculom) {
        if (empty($nmvinculom)) {
            throw new \Exception("Campo Obrigatório: 'Nome'");
        }

        $this->nmvinculom = $nmvinculom;
    }

    /**
     * @param $nrtipovinculom
     * @throws \Exception
     * @return \Folha\Entities\MigVinculom
     */
    public function setNrtipovinculom ($nrtipovinculom) {
        if (!empty($nrtipovinculom)){
            $tipovinculo = \Folha\Entities\GpeTipovinculom::getEntityByWhere('NRTIPOVINCULOM = ?', array($nrtipovinculom));

            if (is_object($tipovinculo)) {
                $this->nrtipovinculom = $nrtipovinculom;
            } else {
                $this->nrtipovinculom = self::DEFAULT_TP_VINCULO;
    //            throw new \Exception("O código do Tipo de Vínculo '$tipovinculo' não foi encontrado.");
            }
        } else {
            $this->nrtipovinculom = self::DEFAULT_TP_VINCULO;
        }
    }

    /**
     * @param $nrtpadmissao
     * @return \Folha\Entities\MigVinculom
     */
    public function setNrtpadmissao ($nrtpadmissao) {
        if (!empty($nrtpadmissao)) {
            $nrorg = ProjetoImportacao::getNrorgPadrao();
            $tipoadmissao = \Folha\Entities\FpaTpadmissao::getEntityByWhere('NRTPADMISSAO = ? AND NRORG = ?', array($nrtpadmissao, $nrorg));

            if (is_object($tipoadmissao)) {
                $this->nrtpadmissao = $nrtpadmissao;
            } else {
                $this->nrtpadmissao = self::DEFAULT_TP_ADMISSAO;
    //            throw new \Exception("O código do Tipo de Admissão '$tipoadmissao' não foi encontrado.");
            }
        } else {
            $this->nrtpadmissao = self::DEFAULT_TP_ADMISSAO;
        }
    }

    /**
     * @param $nrdiascontratoexp
     * @return \Folha\Entities\MigVinculom
     */
    public function setNrdiascontratoexp ($nrdiascontratoexp) {
        if (empty($nrdiascontratoexp)) {
            $nrdiascontratoexp = self::DEFAULT_NRDIAS;
        }
        $this->nrdiascontratoexp = $nrdiascontratoexp;
    }

    /**
     * @param $nrdiascontratoexppro
     * @return \Folha\Entities\MigVinculom
     */
    public function setNrdiascontratoexppro ($nrdiascontratoexppro) {
        if (empty($nrdiascontratoexppro)) {
            $nrdiascontratoexppro = self::DEFAULT_NRDIAS;
        }
        $this->nrdiascontratoexppro = $nrdiascontratoexppro;
    }


    /**
     * @param mixed $dtadmissaovinc
     * @throws \Exception
     * @return \Folha\Entities\MigVinculom|void
     */
    public function setDtadmissaovinc($dtadmissaovinc)
    {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtadmissaovinc)) {
            throw new \Exception("Campo Obrigatório: Data '$dtadmissaovinc' com formato inválido ou em branco.");
        }

        if (!($dtadmissaovinc instanceof \DateTime)) {
            $dtadmissaovinc = DateUtil::getDataDeString($dtadmissaovinc);
        }

        $this->dtadmissaovinc = $dtadmissaovinc;
    }

    /**
     * @param mixed $dtaposentafgts
     * @return \Folha\Entities\MigVinculom|void
     */
    public function setDtaposentafgts($dtaposentafgts)
    {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtaposentafgts)) {
            if (!empty($dtaposentafgts)) {
                throw new \Exception("Dt. Aposentadoria FGTS: Data '$dtaposentafgts' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtaposentafgts instanceof \DateTime)) {
            $dtaposentafgts = DateUtil::getDataDeString($dtaposentafgts);
        }

        $this->dtaposentafgts = $dtaposentafgts;
    }

    /**
     * @param mixed $dtavisoprevio
     */
    public function setDtavisoprevio($dtavisoprevio)
    {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtavisoprevio)) {
            if (!empty($dtavisoprevio)) {
                throw new \Exception("Dt. Aviso Prévio: Data '$dtavisoprevio' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtavisoprevio instanceof \DateTime)) {
            $dtavisoprevio = DateUtil::getDataDeString($dtavisoprevio);
        }

        $this->dtavisoprevio = $dtavisoprevio;
    }

    /**
     * @param mixed $dtavisotrct
     */
    public function setDtavisotrct($dtavisotrct)
    {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtavisotrct)) {
            if (!empty($dtavisotrct)) {
                throw new \Exception("Dt. Aviso TRCT: Data '$dtavisotrct' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtavisotrct instanceof \DateTime)) {
            $dtavisotrct = DateUtil::getDataDeString($dtavisotrct);
        }

        $this->dtavisotrct = $dtavisotrct;
    }

    /**
     * @param mixed $dtexamemedico
     */
    public function setDtexamemedico($dtexamemedico)
    {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtexamemedico)) {
            if (!empty($dtexamemedico)) {
                throw new \Exception("Dt. Exame Médico: Data '$dtexamemedico' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtexamemedico instanceof \DateTime)) {
            $dtexamemedico = DateUtil::getDataDeString($dtexamemedico);
        }

        $this->dtexamemedico = $dtexamemedico;
    }

    /**
     * @param mixed $dtinclusao
     */
    public function setDtinclusao($dtinclusao)
    {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtinclusao)) {
            throw new \Exception("Data '$dtinclusao' com formato inválido ou em branco.");
        }

        if (!($dtinclusao instanceof \DateTime)) {
            $dtinclusao = DateUtil::getDataDeString($dtinclusao);
        }

        $this->dtinclusao = $dtinclusao;
    }

    /**
     * @param mixed $dtopcaofgts
     */
    public function setDtopcaofgts($dtopcaofgts)
    {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtopcaofgts)) {
            if (!empty($dtopcaofgts)) {
                throw new \Exception("Dt. Opção FGTS: Data '$dtopcaofgts' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtopcaofgts instanceof \DateTime)) {
            $dtopcaofgts = DateUtil::getDataDeString($dtopcaofgts);
        }

        $this->dtopcaofgts = $dtopcaofgts;
    }

    /**
     * @param mixed $dtprimadmissao
     */
    public function setDtprimadmissao($dtprimadmissao)
    {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtprimadmissao)) {
            if (!empty($dtprimadmissao)) {
                throw new \Exception("Dt. Primeira Admissão: Data '$dtprimadmissao' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtprimadmissao instanceof \DateTime)) {
            $dtprimadmissao = DateUtil::getDataDeString($dtprimadmissao);
        }

        $this->dtprimadmissao = $dtprimadmissao;
    }

    /**
     * @param mixed $dtrescisaovinc
     */
    public function setDtrescisaovinc($dtrescisaovinc)
    {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtrescisaovinc)) {
            if (!empty($dtrescisaovinc)) {
                throw new \Exception("Dt. Rescisão: Data '$dtrescisaovinc' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtrescisaovinc instanceof \DateTime)) {
            $dtrescisaovinc = DateUtil::getDataDeString($dtrescisaovinc);
        }

        $this->dtrescisaovinc = $dtrescisaovinc;
    }

    /**
     * @param mixed $dtultatu
     */
    public function setDtultatu($dtultatu)
    {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtultatu)) {
            throw new \Exception("Data '$dtultatu' com formato inválido ou em branco.");
        }

        if (!($dtultatu instanceof \DateTime)) {
            $dtultatu = DateUtil::getDataDeString($dtultatu);
        }

        $this->dtultatu = $dtultatu;
    }

    /**
     * @param $nrvinculoempreg
     * @return \Folha\Entities\MigVinculom
     */
    public function setNrvinculoempreg ($nrvinculoempreg)
    {
        if (!empty($nrvinculoempreg)) {
            $nrorg = ProjetoImportacao::getNrorgPadrao();
            $vinculoempreg = \Folha\Entities\GpeVinculoempreg::getObjectByWhere('NRVINCULOEMPREG = ? AND NRORG = ?', array($nrvinculoempreg, $nrorg));

            if (is_object($vinculoempreg)) {
                $this->nrvinculoempreg = $vinculoempreg;
            } else {
                $this->nrvinculoempreg = null;
            }
        } else {
            $this->nrvinculoempreg = null;
        }
    }

    /**
     * @param $nrmotivoresc
     * @return \Folha\Entities\MigVinculom
     */
    public function setNrmotivoresc ($nrmotivoresc)
    {
        if (!empty($nrmotivoresc)) {
            $nrmotivoresc = preg_replace(self::EXPRESSAO_NUMEROS, "", $nrmotivoresc);

            $nrorg = ProjetoImportacao::getNrorgPadrao();
            $motivoresc = \Folha\Entities\GpeMotivoresc::getObjectByWhere('NRMOTIVORESC = ? AND NRORG = ?', array($nrmotivoresc, $nrorg));

            if (is_object($motivoresc)) {
                $this->nrmotivoresc = $nrmotivoresc;
            } else {
                $this->nrmotivoresc = null;
            }
        } else {
            $this->nrmotivoresc = null;
        }
    }
}